const NodeCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/lib/NodeCollection.js');
const TrainingDateCollection = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainingDateCollection.js');
const TrainDateSelector = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/trainDateSelector.js');
const PeriodsDay = window.nguiAsyncUrlImport('/service/intelli/v1/filedb/scripts/maxpermissiblevalues/components/periodsDay.js');

window.ngui(() => {
    return {        
        emits: [
            'form:submit',
        ],
        props: [
            'payload', 
        ],
        methods: {
            setDependentNode(node) {
                this.dependentNode = node;
                this.dependentNodeCollection = [node];
            },
            regionOptionsHasCountry() {
                return Object.keys(this.holidayRegionOptions).includes(this.configModel.holidayLocation.country);
            },
            getMonthDelta(interval) {
                return interval < 0 ? 36 : 3;
            },
            togglePeriodsDayVisibility(trainMethod) {
                this.periodsDayClass = trainMethod === 'movingWindows' ? '' : 'hidden';
            },
            resetDates() {
                this.configModel.measureDates = [];
                this.configModel.trainingDates = [];
                this.localTrainDates = [];
            },
            hasExplanatoryVariables() {
                let hasExplanatoryVariables = false
                if (this.configModel.explanatoryDateTimeVariables) {
                    if (this.configModel.explanatoryDateTimeVariables.length > 0) {
                        hasExplanatoryVariables = true
                    }
                }

                const explVars = this.independentNodes.filter((node) => {
                    if ('purposes' in node) {
                        return node.purposes.includes('computeThresholds') || false;
                    }
                    return false;
                });
                if (explVars.length > 0) {
                    hasExplanatoryVariables = true;
                }

                if (this.configModel.holidayLocation) {
                    if (this.configModel.holidayLocation.country !== '') {
                        hasExplanatoryVariables = true
                    }
                }                
                return hasExplanatoryVariables;
            },
            fetchData(node) {
                return new Promise((resolve, reject) => {
                    if (this.configModel.trainingDates.length === 0) {
                        return reject('Training dates must be set');
                    }

                    if (this.configModel.interval === undefined) {
                        return reject('Interval must be set');
                    }

                    const get = this.configModel.trainingDates.map((date) => {
                        return {
                                path: node.pmModel.path,
                                showExtInfos: ['name', 'unit', 'accType'],    
                                histData: {
                                    interval: this.configModel.interval,
                                    start: date.start,
                                    end: date.end,
                                    interpolateTemplate: node.interpolation.template,
                                }                                    
                            }                        
                    });

                    const payload = {
                        instanceUuid: node.pmModel.instanceUuid,
                        payload: {
                            get,
                        }
                    };

                    this.ctx.dms.toService(node.pmModel.processManagerUuid).post(payload)
                    .then((response) => {
                        let histData = [];
                        response.payload.get.forEach((get) => {
                            histData = histData.concat(get.histData);
                        });
                        resolve(histData);
                    })
                    .catch((err) => {
                        reject(err);
                    });   
                });  
            },
            findObjectsByMatchingKeys(list1, list2, list3 = undefined) {
                if (!Array.isArray(list1) || !Array.isArray(list2)) {
                    this.ctx.log.error("Invalid input: list1 and list2 must be arrays.");
                    return { list1: [], list2: [], list3: [] };
                }

                // Get matching keys between lists
                const keys1 = list1.map(obj => Object.keys(obj)[0]);
                const keys2 = list2.map(obj => Object.keys(obj)[0]);
                let commonKeys = keys1.filter(key => keys2.includes(key));

                if (list3) {
                    if (!Array.isArray(list3)) {
                        this.ctx.log.error("Invalid input: list1 and list2 must be arrays.");
                        return { list1: [], list2: [], list3: [] };
                    }                    
                    const keys3 = list3.map(obj => Object.keys(obj)[0]);
                    commonKeys = commonKeys.filter(key => keys3.includes(key));
                }

                // Filter each list to only include objects with common keys
                const filtered1 = list1.filter(obj => commonKeys.includes(Object.keys(obj)[0]));
                const filtered2 = list2.filter(obj => commonKeys.includes(Object.keys(obj)[0]));
                const filtered3 = list3 ? list3.filter(obj => commonKeys.includes(Object.keys(obj)[0])) : [];

                return {
                    x: filtered1,
                    y: filtered2,
                    z: filtered3
                };
            },
            handleCheckboxUpdate(){

                // init data
                this.dataAnalysis.scatter.xData  = [];
                this.dataAnalysis.scatter.yData  = [];
                this.dataAnalysis.scatter.zData  = [];
                this.dataAnalysis.scatter.xLabel = '';
                this.dataAnalysis.scatter.yLabel =  '';
                this.dataAnalysis.scatter.zLabel =  '';
                this.dataAnalysis.scatter.seriesName =  '';                

                // handle checkboxes states according to number of dimensions
                const maxAllowedActiveCheckboxes = 3;
                const activeCheckboxes = this.dataAnalysis.scatter.checkboxes.filter((checkbox) => checkbox.checked === true);
                if(activeCheckboxes.length >= maxAllowedActiveCheckboxes) {
                    this.dataAnalysis.scatter.checkboxes = this.dataAnalysis.scatter.checkboxes.map(checkbox => {
                        if (checkbox.checked === false) {
                            return { ...checkbox, disabled: true };
                        }
                        return checkbox;
                    });                    
                } else {
                        // Clear chart
                        this.dataAnalysis.scatter.chartSeries[0].data = [];

                        // Activate checkboxes
                        this.dataAnalysis.scatter.checkboxes = this.dataAnalysis.scatter.checkboxes.map(checkbox => {
                        return { ...checkbox, disabled: false };
                    });
                }

                if (activeCheckboxes.length >= 2) {
                    const nodes = activeCheckboxes.map((checkbox) => this.fetchData(checkbox.node));
                    Promise.all(nodes)
                    .then((histDataList) => {
                        const lists = this.findObjectsByMatchingKeys(histDataList[0], histDataList[1], histDataList[2]);
                        
                        const getValue = (data) => Object.values(data)[0];
                        this.dataAnalysis.scatter.xData = lists.x.map(getValue);
                        this.dataAnalysis.scatter.yData = lists.y.map(getValue);

                        const labelX = this.ctx.intelli.getNodeChartName(activeCheckboxes[0].node);                
                        const labelY = this.ctx.intelli.getNodeChartName(activeCheckboxes[1].node);

                        this.dataAnalysis.scatter.xLabel = labelX;
                        this.dataAnalysis.scatter.yLabel = labelY;
                        this.dataAnalysis.scatter.seriesName = `${labelX} - ${labelY}`;

                       let labelZ;
                       if (lists.z.length > 0) {
                            this.dataAnalysis.scatter.zData  = lists.z.map(getValue);
                            labelZ = this.ctx.intelli.getNodeChartName(activeCheckboxes[2].node);
                            this.dataAnalysis.scatter.zLabel = labelZ;
                            this.dataAnalysis.scatter.seriesName = `${labelX} - ${labelY} - ${labelZ}`;
                        }
                    })
                    .catch((err) => {
                        this.ctx.log.error(err);
                    })                    
                }
            },
            persistConfigModel() {
                return new Promise((resolve, reject) => {
                    this.ctx.filedb.insert('app/BAI/configModel/'+ this.configModel.uuid + '.json', this.configModel)
                    .then(() => {
                        resolve(true);
                    })
                    .catch((err) => {
                        reject(err);
                    })
                });
            },                 
        },
        // The returned object can be accessed with this within any method...
        // Do not use this.ctx within this function because it is not yet created.
        data() {
            return {
                dataAnalysis: {
                    activeTab: 'histogram',
                    scatter: {
                        checkboxes: [],
                        xData: [],                        
                        yData: [],                        
                        zData: [],
                        xLabel: '',
                        yLabel: '',
                        zLabel: '',
                        seriesName: '',
                    },
                    histogram: {
                        instancesProps: [],
                    }
                },
                algorithm: '',
                configModelLoaded: false,
                intervalOptions:  [
                    {label: 'hourly', value: 3600},
                    {label: 'daily', value: -1},
                    // {label: 'weekly', value: -2},
                ],
                dependentNodeCollection: [], // interface var for collection
                dependentNode: {},
                independentNodes: [],
                // localTrainMethod - localTrainDates are buffers
                // to persist data on submit only
                localTrainMethod: 'movingWindows',
                localTrainDates: [],
                periodsDayClass: '',
                explanatoryDateTimeOptions: [
                    {label: 'hod', value: 'hod'},
                    {label: 'dow', value: 'dow'},
                    {label: 'woy', value: 'woy'},
                    {label: 'month', value: 'month'},                    
                    {label: 'quarter', value: 'quarter'},                    
                    {label: 'Mo2Sa-So', value: 'Mo2Sa-So'},
                    {label: 'Mo2Fr-Sa-So', value: 'Mo2Fr-Sa-So'},        
                    {label: 'Mo2Fr-Sa2So', value: 'Mo2Fr-Sa2So'},
                ],
                holidayCountryOptions: [
                    {label: '', value: ''},
                    {label: 'switzerland', value: 'CH'},
                    {label: 'germany', value: 'DE'},
                    {label: 'luxembourg', value: 'LU'},
                    {label: 'austria', value: 'AT'},
                    {label: 'netherlands', value: 'NL'},
                ],
                holidayRegionOptions: {
                    'default': [
                        {label: '', value: ''},
                    ],
                    'CH': [
                        {label: '', value: ''},
                        {label: 'ZH', value: 'ZH'},
                        {label: 'BE', value: 'BE'},
                        {label: 'LU', value: 'LU'},
                        {label: 'UR', value: 'UR'},
                        {label: 'SZ', value: 'SZ'},
                        {label: 'OW', value: 'OW'},
                        {label: 'NW', value: 'NW'},
                        {label: 'GL', value: 'GL'},
                        {label: 'ZG', value: 'ZG'},
                        {label: 'FR', value: 'FR'},
                        {label: 'SO', value: 'SO'},
                        {label: 'BS', value: 'BS'},
                        {label: 'BL', value: 'BL'},
                        {label: 'SH', value: 'SH'},
                        {label: 'AR', value: 'AR'},
                        {label: 'AI', value: 'AI'},
                        {label: 'SG', value: 'SG'},
                        {label: 'GR', value: 'GR'},
                        {label: 'AG', value: 'AG'},
                        {label: 'TG', value: 'TG'},
                        {label: 'VD', value: 'VD'},
                        {label: 'TI', value: 'TI'},
                        {label: 'VS', value: 'VS'},
                        {label: 'NE', value: 'NE'},
                        {label: 'GE', value: 'GE'},
                        {label: 'JU', value: 'JU'},
                    ],
                    'DE': [
                        {label: '', value: ''},
                        {label: 'BW', value: 'BW'},
                        {label: 'BY', value: 'BY'},
                        {label: 'BE', value: 'BE'},
                        {label: 'BB', value: 'BB'},
                        {label: 'HB', value: 'HB'},
                        {label: 'HH', value: 'HH'},
                        {label: 'HE', value: 'HE'},
                        {label: 'MV', value: 'MV'},
                        {label: 'NI', value: 'NI'},
                        {label: 'NW', value: 'NW'},
                        {label: 'RP', value: 'RP'},
                        {label: 'SL', value: 'SL'},
                        {label: 'SN', value: 'SN'},
                        {label: 'ST', value: 'ST'},
                        {label: 'SH', value: 'SH'},
                        {label: 'TH', value: 'TH'},
                    ]               
                },
                dialog: {
                    chart: false,
                    dataAnalysis: false,
                },
                trainMethodOptions: [
                    {label: 'movingWindows', value: 'movingWindows'},
                    {label: 'isolatedWindows', value: 'isolatedWindows'}                
                ],
                configModel: {
                    uuid: '',
                    task: {
                        name: 'maxpermissiblevalues',
                        algorithm: {
                            method: '',
                            model: '',
                        },
                        metric: undefined,
                    }, 
                    interval: -1,
                    nodes: [],
                    dependentVariables: [],
                    independentVariables: [],
                    explanatoryDateTimeVariables: [],
                    holidayLocation: {
                        country: '',
                        region: '',
                    },
                    minElementsPerBin: 3,
                    trainMethod: 'movingWindows',
                    trainTime: '00:00', 
                    trainingDates: [],
                    measureDates: [],
                    periodsDay: {
                        train: 60,
                        retrain: 20,
                        ignoreEvents: false,
                    },
                    sensitivityThreshold: {
                        upper: 1,
                        lower: 1,
                    },
                    eventCount: {
                        upper: 3,
                        lower: 3,
                    },
                    minErrorDistance: {
                        upper: 0,
                        lower: 0,
                    },
                    trespassing: {
                        upper: true,
                        lower: true,                        
                    },
                    thresholdColor: '#99ff99',
                    expectedValueColor: '#47b547',
                },                        
                modelOptions: [                                      
                    '',
                    'groupingMethod', // Grouping method
                    'AdaBoostRegressor',
                    'BaggingRegressor',
                    'DecisionTreeRegressor',
                    'ExtraTreesRegressor',
                    'GammaRegressor',
                    'GaussianProcessRegressor',
                    'GradientBoostingRegressor',
                    'HistGradientBoostingRegressor',
                    'HuberRegressor',
                    'KNeighborsRegressor',
                    'KernelRidge',
                    'Lars',
                    'LarsCV',
                    'LassoCV',
                    'LassoLarsCV',
                    'LassoLarsIC',
                    'LinearRegression',
                    'LinearSVR',
                    'MLPRegressor',
                    'NuSVR',
                    'OrthogonalMatchingPursuit',
                    'OrthogonalMatchingPursuitCV',
                    'PiecewiseRegressor',                    
                    'RANSACRegressor',
                    'RadiusNeighborsRegressor',
                    'RandomForestRegressor',
                    'RidgeCV',
                    'SVR',
                    'TheilSenRegressor',
                    'XGBoost',
                ] 
            }       
        },
        beforeMount() {
            this.widget = this.payload.widget;
            const now = this.ctx.moment.moment();
            const defaultTrainingDates = {
                start: '',
                end: now.set({minute:0, second:0, millisecond:0}).format()
            }
            const months = this.getMonthDelta(this.configModel.interval);
            defaultTrainingDates.start = now.subtract(months, 'months').format();

            this.configModel.trainingDates.push(defaultTrainingDates);
            this.dependentNode = this.ctx.intelli.initNode();
            this.dependentNode.uuid = this.ctx.uuidv4();
                        
            // Get config model
            this.ctx.filedb.read('app/BAI/configModel/'+ this.widget.configModelUuid + '.json', true)
            .then((response) => {
                const configModel = response.data.data;
                this.configModel = {...this.configModel, ...configModel};

                const dependentNodes = this.configModel.nodes.filter((node) => {
                    return (this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });
                if (dependentNodes.length > 0) {
                    this.setDependentNode(dependentNodes[0]);                    
                }                

                this.independentNodes = this.configModel.nodes.filter((node) => {
                    // Filter all remaining nodes except dependent node
                    // Use dependentVarialbes since independentVariables contains only the ones with enabled purpose
                    return !(this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`));
                });

                this.localTrainMethod = this.configModel.trainMethod;
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                

                this.algorithm = this.configModel.task.algorithm.model;
                // Fix (#54157) for groupingMethod
                if ( this.configModel.task.algorithm.method === 'thresholdsGrouping') {
                    this.algorithm = 'groupingMethod';
                }

                this.configModelLoaded = true;
            })
            .catch(() => {
                this.togglePeriodsDayVisibility(this.configModel.trainMethod);                    
            });                
        },
        // Define elements within the ui configuration...
        nodes() {
            return [
                {
                    type: 'NgDialog',
                    props: {
                        title: 'analyseData',
                        maximizing: true,
                        modelValue: this.dialog.dataAnalysis,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.dataAnalysis = value;
                        },
                        'onBefore:show': () =>{
                            //prepare checkboxes for scatter 
                            this.dataAnalysis.scatter.checkboxes = this.configModel.nodes.map((node) => {
                                return {
                                    node,
                                    label: this.ctx.intelli.getNodeChartName(node),
                                    checked: false,
                                    disabled: false,
                                };
                            });
                        },
                        'onHide': () => {
                            this.dataAnalysis.scatter.chartSeries[0].data = [];
                        },
                        width: '1000px',
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgTabs',
                                    props: {
                                        modelValue: this.dataAnalysis.activeTab,
                                        'onUpdate:modelValue': (value) => {
                                            this.dataAnalysis.activeTab = value;
                                        },
                                        tabs: [
                                            {
                                               name: 'histogram',
                                               label: 'Histogram',
                                            },
                                            {
                                               name: 'scatter',
                                               label: 'Scatter',
                                            },
                                        ],
                                    },                    
                                    children: {
                                        histogram: () => {
                                            let vNodes = [
                                                {
                                                    type: 'NgSpacing',
                                                    props: {
                                                        mt: 'sm',
                                                        mb: 'lg',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            let vNodes = [
                                                                {
                                                                    type: 'NgText',
                                                                    props: {
                                                                        text: 'trainingDates', 
                                                                        type: 'subtitle1',
                                                                        look: 'weight-medium',
                                                                    },
                                                                }
                                                            ];
                                                            vNodes = vNodes.concat(this.configModel.trainingDates.map((dates) => {
                                                                return {
                                                                    type: 'NgText',
                                                                    props: {                                                        
                                                                        text: `${this.ctx.moment.moment(dates.start).format('DD.MM.YYYY')} - ${this.ctx.moment.moment(dates.end).format('DD.MM.YYYY')}`,
                                                                    }
                                                                };
                                                            }));
                                                            return vNodes;
                                                        }
                                                    }
                                                }, 

                                            ];
                                            
                                            this.dataAnalysis.histogram.instancesProps.forEach((instance) => {
                                                vNodes.push({
                                                    type: 'NgHistogram',
                                                    props: {
                                                        title: instance.title,
                                                        data: instance.data,
                                                    }
                                                });
                                                vNodes.push({
                                                    type: 'NgSeparator',
                                                    props: {
                                                        spaced: 'xl',
                                                    },
                                                });
                                            });
                                            return vNodes;
                                        },
                                        scatter: () => {

                                            let vNodes = [
                                                {
                                                    type: 'NgSpacing',
                                                    props: {
                                                        mt: 'sm',
                                                        mb: 'lg',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            let vNodes = [
                                                                {
                                                                    type: 'NgText',
                                                                    props: {
                                                                        text: 'trainingDates', 
                                                                        type: 'subtitle1',
                                                                        look: 'weight-medium',
                                                                    },
                                                                }
                                                            ];
                                                            vNodes = vNodes.concat(this.configModel.trainingDates.map((dates) => {
                                                                return {
                                                                    type: 'NgText',
                                                                    props: {                                                        
                                                                        text: `${this.ctx.moment.moment(dates.start).format('DD.MM.YYYY')} - ${this.ctx.moment.moment(dates.end).format('DD.MM.YYYY')}`,
                                                                    }
                                                                };
                                                            }));
                                                            return vNodes;
                                                        }
                                                    }
                                                }, 

                                            ];

                                            vNodes = vNodes.concat(this.dataAnalysis.scatter.checkboxes.map((item) => {
                                                return {
                                                    type: 'NgFormInput',                                
                                                    props: {
                                                        disable: item.disabled,
                                                        label: item.label,
                                                        type: 'checkbox',
                                                        modelValue: item.checked,
                                                        'onUpdate:modelValue': (value) => {
                                                            item.checked = value;
                                                            this.handleCheckboxUpdate();
                                                        },                                                                     
                                                    },    
                                                }
                                            }));

                                            vNodes.push({
                                                type: 'NgScatter',
                                                props: {
                                                    x: this.dataAnalysis.scatter.xData,
                                                    y: this.dataAnalysis.scatter.yData,
                                                    z: this.dataAnalysis.scatter.zData,
                                                    xAxisLabel: this.dataAnalysis.scatter.xLabel,
                                                    yAxisLabel: this.dataAnalysis.scatter.yLabel,
                                                    zAxisLabel: this.dataAnalysis.scatter.zLabel,
                                                    seriesName: this.dataAnalysis.scatter.seriesName,
                                                }
                                            });
                                            
                                            return vNodes;
                                        }
                                    }
                                },                                 
                            ];
                        }
                    }
                },
                {
                    type: 'NgDialog',
                    props: {
                        title: '',
                        maximizing: true,
                        modelValue: this.dialog.chart,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.chart = value;
                        },                        
                        width: '1000px',
                        medium: false,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                    },
                    children: {
                        default: () => {                            
                            return [   
                                {
                                    type: TrainDateSelector,
                                    props: {
                                        trainMethodOptions: this.trainMethodOptions,
                                        trainMethod: this.configModel.trainMethod,
                                        trainDates: this.configModel.trainingDates,
                                        nodeCollection: this.configModel.nodes,
                                        interval: this.configModel.interval,
                                        intervalOptions: this.intervalOptions,                        
                                        'onUpdate:interval': (value) => {
                                            this.configModel.interval = value;
                                        },                                        
                                        'onUpdate:nodeCollection': (values) => {
                                            this.configModel.nodes = values;
                                        },
                                        'onUpdate:trainDates': (trainDates) => {
                                            this.localTrainDates = trainDates;                                                   
                                        },                                   
                                        'onUpdate:trainMethod': (value) => {
                                            this.localTrainMethod = value;
                                            this.togglePeriodsDayVisibility(value);
                                        },
                                        'onSubmit': () => {
                                            // Set data on save only                                            
                                            this.configModel.trainMethod = this.localTrainMethod;
                                            this.configModel.trainingDates = this.localTrainDates;

                                            this.persistConfigModel()
                                            .catch((err) => {
                                                this.ctx.log.error(err);
                                            })
                                            .finally(() => {
                                                this.dialog.chart = false;
                                            });    
                                        }
                                    }
                                }
                            ];
                        }
                    }
                },                        
                {
                    type: 'NgForm',                
                    props: {
                        greedy: true,
                        submitLabel: 'computeThresholds',
                        'data-cy': 'computeThresholdsForm',
                        key: this.configModelLoaded,
                        onSubmit: () =>  {
                            this.configModel.nodes = [...this.independentNodes, this.dependentNode];

                            // Fix (#54157) for groupingMethod
                            if (this.algorithm === 'groupingMethod') {
                                this.configModel.task.algorithm.method = 'thresholdsGrouping';
                                this.configModel.task.algorithm.model = '';
                            }

                            // (#54012) Reset measure dates for rollingWindows
                            this.configModel.measureDates = [];

                            // Set transform dates from latest end date to today
                            if (this.configModel.trainMethod === 'isolatedWindows') {
                                const measureStartDate = this.ctx.intelli.getMeasureStartDate(this.configModel.trainingDates, this.configModel.interval);
                                const today = this.ctx.moment.moment().format();
                                this.configModel.measureDates = [{
                                    'start': measureStartDate,
                                    'end': today
                                }];
                            }
                            
                            if (this.configModel.holidayLocation) {
                                if (this.configModel.holidayLocation.country !== '') {
                                    this.configModel.independentVariables.push('holiday');
                                }
                            }

                            // set uuids
                            this.configModel.nodes = this.configModel.nodes.map((node) => {
                                if (!node.uuid) {
                                    node.uuid = this.ctx.uuidv4();
                                }
                                return node;
                            });
                            
                            const nodes = this.configModel.nodes?.map((node) => {
                                return {
                                    uuid: node.uuid,                               
                                    dependentVariable: (this.configModel.dependentVariables.includes(`${node.pmModel.processUuid}_${node.pmModel.path}`)),
                                    interpolation: node.interpolation,
                                    bins: node.bins,
                                    pmModel: node.pmModel,
                                    extInfos: node.extInfos,
                                    chartOptions: node.chartOptions,
                                };
                                });

                            const trainingDates = this.configModel.trainingDates.map((dates) => {
                                let end = dates.end;
                                if (!dates.end) {
                                    end = this.ctx.moment.getLocalizedNow().format()
                                }
                                return {
                                    start: dates.start,
                                    end
                                }
                            });

                            if (this.widget.name === '') {
                                this.widget.name = this.dependentNode['pmModel']['path'];
                                if (this.dependentNode['extInfos']['name'] !== '') {
                                    this.widget.name = `${this.dependentNode['extInfos']['name']} - ${this.dependentNode['pmModel']['path']}`;
                                }
                            }

                            const payload = {
                                scriptType: 'maxpermissiblevalues',
                                scriptFile: 'compute_thresholds.py',
                                scriptArguments: {
                                    configModelUuid: this.configModel.uuid,
                                    trainingDates: trainingDates,
                                    measureDates: this.configModel.measureDates,
                                    dependentNodeName: this.ctx.intelli.getNodeChartName(this.dependentNode)
                                },
                                dates: trainingDates.concat(this.configModel.measureDates),
                                nodes,
                                holidayLocation: this.configModel.holidayLocation,
                                interval: this.configModel.interval, 
                                outputUuid: this.widget.outputUuid,
                            }

                            Promise.all([this.persistConfigModel(), this.ctx.filedb.insert('app/BAI/widgets/'+ this.widget.uuid + '.yaml', this.widget)])
                            .then(() => {
                                this.$emit('form:submit', payload);
                            })
                            .catch((err) => {                                    
                                this.ctx.log.error(err);
                            });

                        },
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'observedDatapoint',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: NodeCollection,
                                                                    props: {
                                                                        modelValue: this.dependentNodeCollection,
                                                                        purposeOptions: [],
                                                                        selectedAlgo: this.configModel.task.algorithm,
                                                                        maxItems: 1,
                                                                        'onUpdate:modelValue': (nodes) => {                                                                                                                                                        
                                                                            if (nodes.length > 0) {
                                                                                const dependentNode = nodes[0];
                                                                                this.setDependentNode(dependentNode);

                                                                                // (#54120) Render dependent node left handside
                                                                                dependentNode.chartOptions.y.opposite = false;
                                                                                this.configModel.dependentVariables = [`${dependentNode['pmModel']['processUuid']}_${dependentNode['pmModel']['path']}`];

                                                                                // keep this.configModel.nodes update to date for trainDateSelector & data analysis
                                                                                this.configModel.nodes = [dependentNode, ...this.independentNodes];
                                                                                this.persistConfigModel()
                                                                                .catch((err) => {
                                                                                    this.ctx.log.error(err);
                                                                                });       
                                                                            }
                                                                        },                                                                          
                                                                        datapointListRules: [
                                                                            `
                                                                            (function() {
                                                                                let rv = $t('ruleMinElement', {'total': 1});
                                                                                if (props.modelValue.length > 0) {
                                                                                    rv = true;
                                                                                }
                                                                                return rv;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),     
                                                                        ]
                                                                    },
                                                                },                                                           
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [                                                    
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'explanatoryVariables',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: NodeCollection,
                                                                    props: {
                                                                        modelValue: this.independentNodes,
                                                                        purposeOptions: ['computeThresholds'],
                                                                        collectionLabel: 'datapointList',
                                                                        selectedAlgo: this.configModel.task.algorithm,
                                                                        hasExplanatoryVariables: this.hasExplanatoryVariables(),
                                                                        'onUpdate:modelValue': (independentNodes) => {
                                                                            this.independentNodes = independentNodes;

                                                                            const independentVariables = independentNodes.filter((node) => ('purposes' in node && node['purposes'].includes('computeThresholds')))
                                                                            .map((node) => {
                                                                                return `${node['pmModel']['processUuid']}_${node['pmModel']['path']}`;
                                                                            });                                                                            
                                                                            this.configModel.independentVariables = [ ...independentVariables, ...this.configModel.explanatoryDateTimeVariables];

                                                                            // keep this.configModel.nodes update to date for trainDateSelector
                                                                            this.configModel.nodes = [this.dependentNode, ...independentNodes];

                                                                            this.persistConfigModel()
                                                                            .catch((err) => {
                                                                                this.ctx.log.error(err);
                                                                            });
                                                                        },                                                                          
                                                                        datapointListRules: [
                                                                            `
                                                                            (function() {
                                                                                if (!attrs.hasExplanatoryVariables) {
                                                                                    return $t('ruleAtLeastOneExplanatoryVariableWithPurpose');
                                                                                }                                                                                
                                                                                return true;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),                                                                             
                                                                            `
                                                                            (function() {
                                                                                if (attrs.selectedAlgo.method === 'thresholdsGrouping') {
                                                                                    const errors = val.filter((node) => {
                                                                                        if (node['extInfos']['accType'] === 'double64') {
                                                                                            if ('bins' in node) {
                                                                                                if ('base' in node['bins'] === false || 'step' in node['bins'] === false) {
                                                                                                    return true;
                                                                                                }
                                                                                                if ([node['bins']['base'], node['bins']['step']].includes(undefined)) {
                                                                                                    return true;
                                                                                                }
                                                                                            }
                                                                                        }                                                                                     
                                                                                    });     
                                                                                    if (errors.length > 0) {
                                                                                        return $t('ruleAlgoNeedsBinning');
                                                                                    }
                                                                                }
                                                                                return true;
                                                                            })();
                                                                            `.replace(/(\r\n|\n|\r)/gm, ""),                                                                                                                                             
                                                                        ]
                                                                    },
                                                                },                                                                
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'dateTimeVariables',
                                                                        type: 'select',
                                                                        multiple: true,
                                                                        options: this.explanatoryDateTimeOptions,
                                                                        modelValue: this.configModel.explanatoryDateTimeVariables,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.explanatoryDateTimeVariables = value;
                                                                        },
                                                                        rules: [
                                                                            () => this.hasExplanatoryVariables() || this.$t("ruleAtLeastOneExplanatoryVariableWithPurpose")
                                                                        ]                                                                       
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'holidaysCountry',
                                                                        type: 'select',
                                                                        options: this.holidayCountryOptions,
                                                                        modelValue: this.configModel.holidayLocation.country,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.holidayLocation.country = value;
                                                                            this.configModel.holidayLocation.region = '';
                                                                        }                                                                    
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',                                
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'holidaysRegion',
                                                                        disable: !this.regionOptionsHasCountry(),
                                                                        type: 'select',
                                                                        options: this.holidayRegionOptions[this.regionOptionsHasCountry() ? this.configModel.holidayLocation.country : 'default'],
                                                                        modelValue: this.configModel.holidayLocation.region,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.holidayLocation.region = value;
                                                                        },
                                                                        rules: [
                                                                            (val) => {
                                                                                if (this.regionOptionsHasCountry()) {
                                                                                    return val ? true : this.$t('ruleNotEmpty');
                                                                                }
                                                                                return true;
                                                                            }
                                                                        ]                                                                                                                                         
                                                                    },
                                                                },                                                                                                                        
                                                                // {
                                                                //     type: 'NgFormInput',
                                                                //     props: {
                                                                //         col: 6,
                                                                //         label: 'minElementsPerBin',
                                                                //         type: 'number',
                                                                //         modelValue: this.configModel.minElementsPerBin,
                                                                //         'onUpdate:modelValue': (value) => {
                                                                //             this.configModel.minElementsPerBin = value;
                                                                //         },
                                                                //         rules: [(val) => val !== "" || this.$t("ruleNotEmpty")]                                                                          
                                                                //     },
                                                                // },                                                                                                                               
                                                            ];
                                                        }
                                                    }
                                                },  
                                            ]
                                        }
                                    }                                    
                                },
                                {
                                    type: 'div',
                                    props: {
                                        class: 'row col-12',
                                    },
                                    children: {
                                        default: (slot) => {                            
                                            return [
                                                {
                                                    type: 'NgActionButton',
                                                    props: {
                                                        fullWidth: true,
                                                        color: this.dependentNodeCollection.length === 0 && this.independentNodes.length === 0 ? 'grey' : 'primary',
                                                        disable:this.dependentNodeCollection.length === 0 && this.independentNodes.length === 0  ? true : false,
                                                        label: 'analyseData',
                                                        onClick: () => {
                                                            let nodes = [this.dependentNode];
                                                            nodes = nodes.concat(this.independentNodes);

                                                            const fetchPromises = nodes.map((node) => {
                                                                return this.fetchData(node);
                                                            });

                                                            Promise.all(fetchPromises)
                                                            .then((response) => {
                                                                this.dataAnalysis.histogram.instancesProps = response.map((histData, i) => {
                                                                    const data = histData.map((data) => {
                                                                        return Object.values(data)[0];
                                                                    });
                                                                    return {
                                                                        title: this.ctx.intelli.getNodeChartName(nodes[i]),                                                  
                                                                        data,
                                                                    };
                                                                });
                                                                this.dialog.dataAnalysis = true;
                                                            })
                                                            .catch((err) => {
                                                                this.ctx.notify.notify('negative', err);                                                                
                                                                this.ctx.log.error(err);
                                                            });

                                                        }    
                                                    },
                                                },                                                                                    
                                            ]
                                        }
                                    }
                                },                                
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12',
                                        separator: false,
                                        bordered: false,
                                        dense: true,

                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeResolution',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'interval',
                                                                        type: 'select',
                                                                        options: this.intervalOptions,
                                                                        modelValue: this.configModel.interval,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.interval = value;
                                                                            if (this.configModel.trainingDates.length > 0) {
                                                                                const months = this.getMonthDelta(this.configModel.interval);
                                                                                // Set default training dates on time resolution update.
                                                                                if (this.configModel.trainMethod === 'movingWindows') {
                                                                                    let endDate = this.ctx.moment.getLocalizedNow();
                                                                                    if (this.configModel.trainingDates[0].end !== '') {
                                                                                        endDate = this.ctx.moment.getDate(this.configModel.trainingDates[0].end)
                                                                                    }
                                                                                    this.configModel.trainingDates[0].start = endDate.subtract(months, 'months').format();                                            
                                                                                }
                                                                            }
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty')]                                                                             
                                                                    },
                                                                },                                                                   
                                                            ]
                                                        }
                                                    }
                                                }, 
                                            ]
                                        },
                                    }
                                },                                         
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'algorithm',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 6,
                                                                        label: 'model',                                        
                                                                        type: 'select',
                                                                        options: this.modelOptions.map((option) => { 
                                                                            return { label: option, value: option };
                                                                        }),
                                                                        modelValue: this.algorithm,                                                                          
                                                                        'onUpdate:modelValue': (value) => {      
                                                                            this.algorithm = value;                                                                  
                                                                            this.configModel.task.algorithm.method = 'thresholdsRegression';                                           
                                                                            this.configModel.task.algorithm.model = value;

                                                                            if (value === 'groupingMethod') {
                                                                                this.configModel.task.algorithm.method = 'thresholdsGrouping';
                                                                                this.configModel.task.algorithm.model = '';
                                                                            }
                                                                        },
                                                                        rules: [
                                                                            (val) => ((val !== undefined && val !== '')) || this.$t('ruleNotEmpty')
                                                                        ]                                                                                                      
                                                                    },
                                                                },                                                                 
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                                                                                                                                                                                                                                                                                                                                                                              
                                {
                                    type: 'NgFormGroups',                                
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'timeSpans',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [                                                               
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'trainMethod',
                                                                        type: 'select',
                                                                        modelValue: this.configModel.trainMethod,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainMethod = value;
                                                                            this.localTrainMethod = value;
                                                                            this.togglePeriodsDayVisibility(value);
                                                                            this.resetDates();
                                                                        },
                                                                        options: this.trainMethodOptions,
                                                                        rules: [
                                                                            (val) => val !== '' || this.$t('ruleNotEmpty'),
                                                                            () => this.configModel.trainingDates.length > 0 || this.$t('ruleMinElement', {total: 1}),
                                                                        ]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'div',
                                                                    props: {
                                                                        class: 'row col-12',
                                                                    },
                                                                    children: {
                                                                        default: (slot) => {                            
                                                                            return [
                                                                                {
                                                                                    type: 'NgActionButton',
                                                                                    props: {
                                                                                        fullWidth: true,
                                                                                        color: this.dependentNodeCollection.length > 0 ? 'primary' : 'grey',
                                                                                        disable: this.dependentNodeCollection.length > 0 ? false : true,
                                                                                        label: 'visualizeDatapoint',
                                                                                        onClick: () => {
                                                                                            this.dialog.chart = true;
                                                                                        }    
                                                                                    },
                                                                                },                                                                                    
                                                                            ]
                                                                        }
                                                                    }
                                                                },                                                              
                                                                {
                                                                    type: TrainingDateCollection,
                                                                    props: {
                                                                        class: 'col-12',
                                                                        trainMethod: this.configModel.trainMethod,
                                                                        modelValue: this.configModel.trainingDates,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trainingDates = value;
                                                                            this.persistConfigModel()
                                                                            .catch((err) => {
                                                                                this.ctx.log.error(err);
                                                                            });    
                                                                        },                                        
                                                                    },
                                                                },                                                            
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                 
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12 ' + this.periodsDayClass                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'trainingPeriod',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            const trainingPeriodRules = [];
                                                            if (this.configModel.task.algorithm.method === 'thresholdsGrouping') {
                                                                trainingPeriodRules.push(
                                                                    () => {
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('quarter')
                                                                            && this.configModel.periodsDay.train <= 90) {
                                                                                return 'thresholdsGroupingAndQuarterAtLeast90Days';
                                                                        }
                                                                        if (this.configModel.explanatoryDateTimeVariables.includes('month')
                                                                            && this.configModel.periodsDay.train <= 30) {
                                                                                return 'thresholdsGroupingAndMonthAtLeast30Days';
                                                                        }                                    
                                                                        return true;
                                                                    });
                                                            }
                                                            return [
                                                                {
                                                                    type: PeriodsDay,
                                                                    props: {
                                                                        class: 'row',
                                                                        modelValue: this.configModel,
                                                                        renderIgnoreEvents: false,
                                                                        trainingPeriodRules,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel = value;
                                                                        }                                                                     
                                                                    },
                                                                },                                                                
                                                            ];
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'sensitivity',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'upper',
                                                                        type: 'slider',
                                                                        modelValue: this.configModel.sensitivityThreshold.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.upper = value;
                                                                        },
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        reverse: true,
                                                                        required: true,
                                                                        default: 1,                          
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col:12,
                                                                        label: 'lower',
                                                                        type: 'slider',
                                                                        min: 1,
                                                                        max: 100,
                                                                        steps: 1,
                                                                        required: true,
                                                                        default: 1,                                   
                                                                        modelValue: this.configModel.sensitivityThreshold.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.sensitivityThreshold.lower = value;
                                                                        },
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'eventCount',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.upper = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },   
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.eventCount.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.eventCount.lower = value;
                                                                        },
                                                                        rules: [(val) => val !== '' && val !== undefined  || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'minErrorDistance',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.upper = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 || this.$t('ruleAbsolute')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'number',
                                                                        modelValue: this.configModel.minErrorDistance.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.minErrorDistance.lower = value;
                                                                        },
                                                                        step: 'any',
                                                                        rules: [(val) => val !== '' && val !== undefined || this.$t('ruleNotEmpty'), (val) => val >= 0 ||this.$t('ruleAbsolute')]
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:() => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'thresholdTrespassingActivation',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'upper',
                                                                        type: 'toggle',
                                                                        modelValue: this.configModel.trespassing.upper,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trespassing.upper = value;
                                                                        },
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'lower',
                                                                        type: 'toggle',
                                                                        modelValue: this.configModel.trespassing.lower,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.trespassing.lower = value;
                                                                        },
                                                                    },
                                                                },                                                                
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                             
                                {
                                    type: 'NgFormGroups',      
                                    props: {    
                                        class: 'col-12'                                                         
                                    },
                                    children: {
                                        default:(slot) => {
                                            return [
                                                {
                                                    type: 'NgFormGroupItem',                                
                                                    props: {
                                                        title: 'visualization',
                                                    },
                                                    children: {
                                                        default: (slot) => {
                                                            return [
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'thresholdColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.thresholdColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.thresholdColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },
                                                                {
                                                                    type: 'NgFormInput',
                                                                    props: {
                                                                        col: 12,
                                                                        label: 'expectedValueColor',
                                                                        type: 'color',
                                                                        modelValue: this.configModel.expectedValueColor,
                                                                        'onUpdate:modelValue': (value) => {
                                                                            this.configModel.expectedValueColor = value;
                                                                        },
                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')]
                                                                    },
                                                                },                                                                                                                                                                              
                                                            ]
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                }                                                                                               
                            ]                                            
                        },
                    },                   
                }
            ]
        },
    }
});