// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
      'isSetObject',
    ],

    icon: 'send',

    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'dms-path',
                    default: {
                      processManagerUuid: '',
                      instanceUuid: '',
                      processUuid: '',
                      path: '',
                      linkType: 'template',
                    },
                    useLinkType: true,
                    defaultType: 'static',
                    disableLinkTypes: ['none'],
                    'call.disableLinkTypes': 'ctx.attrs.template === undefined ? ["none", "template"] : ["none"]',
                    'call.templateConfig': 'ctx.attrs.template ? ctx.attrs.template.template : undefined',
                    'call.workspaceUuid': 'ctx.props.workspaceUuid',
                    'call.serviceUuids': 'ctx.props.serviceUuids',               
                    label: '',
                    scope: 'link',
                    hint: '',
                    col: 'col-12',
                  },
                  {
                    type: 'radio',
                    default: 'set',
                    label: 'set',
                    scope: 'attrs.method',
                    val: 'set',
                    col: 'col-12',
                  },
                  {
                    type: 'radio',
                    default: 'toggle',
                    label: 'toggle',
                    scope: 'attrs.method',
                    val: 'toggle',
                    col: 'col-12',
                  },
                  {
                    type: 'radio',
                    default: 'set',
                    label: 'pulse',
                    scope: 'attrs.method',
                    val: 'pulse',
                    col: 'col-12',
                  },
                  {
                    'call.hidden': 'return ctx.data.attrs.method !== "pulse"',
                    type: 'number',
                    default: 0,
                    label: 'duration',
                    sideLabel: 'ms',
                    scope: 'attrs.pulseLength',
                    col: 'col-12',
                    min: 0,
                    max: 60000,
                  },
                  {
                    type: 'select',
                    default: 'boolean',
                    label: 'type',
                    scope: 'attrs.valueType',
                    options: [
                      { label: 'string', value: 'string' },
                      { label: 'number', value: 'number' },
                      { label: 'boolean', value: 'boolean' },
                    ],
                    col: 'col-12',
                  },
                  {
                    'call.type': `
                      const valueType = ctx.data.attrs.valueType
                      let type = 'boolean'
                      switch (valueType) {
                        case 'string':
                          type = 'text'
                          break
                        case 'boolean':
                          type = 'checkbox'
                          break
                        case 'number':
                          type = 'number'
                          break
                      }
                      return type
                    `,
                    default: undefined,
                    'call.label': 'return ctx.data.attrs.method === "set" ? "constantValue" : "trueValue"',
                    scope: 'attrs.trueValue',
                    'call.col': 'return ctx.data.attrs.method === "set" ? "col-12" : "col-md-6"',
                    sideSlot: 'link',
                  },
                  {
                    'call.hidden': 'return ctx.data.attrs.method === "set"',
                    'call.type': `
                      const valueType = ctx.data.attrs.valueType
                      let type = 'boolean'
                      switch (valueType) {
                        case 'string':
                          type = 'text'
                          break
                        case 'boolean':
                          type = 'checkbox'
                          break
                        case 'number':
                          type = 'number'
                          break
                      }
                      return type
                    `,
                    default: undefined,
                    label: 'falseValue',
                    scope: 'attrs.falseValue',
                    col: 'col-12 col-md-6',
                    sideSlot: 'link',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        value: false,
        loading: true,
        prevEvent: null,
        firstEvent: null,
        timeoutId: 0,
      }
    },
    
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
      firstEvent: {
        handler() {
          if (this.isSetObject) {
            clearTimeout(this.timeoutId)
             this.timeoutId = setTimeout(() => {
                if (!(this.item.attrs.disable || this.item.attrs.readonly)) {
                  this.emitValue()
                } else {
                  this.ctx.notify.notify('negative', 'noPermission')
                }
             }, 750)           
          }

        },
        deep: true,
      },
    },  
    created() {
      this.item.valueType = ''
      this.value = this.item.attrs.constantValue
      this.panel.on(this.computedObject, '',  this.onEvent)
      this.panel.setPermissions(this.item.rights, this.item)
    },
    
    // Listen on mounted event...
    mounted() {
      this.loading = !!this.isSetObject
      this.item.attrs.icon = !this.isSetObject ? 'send' : '' 
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
      clearTimeout(this.timeoutId)
      this.panel.off(this.computedObject, '', this.onEvent)
    },
	
    methods: {
      getToggleValue() {
        const path = this.computedObject.getPath();
        const { trueValue, falseValue } = this.item.attrs
        const value = this.value === trueValue ? falseValue : trueValue

        return [
          {
            path,
            value,
            tag: {
              cmd: this.computedObject.getIdent(),
            }
          }
        ]
      },

      getSetValue() {
        const path = this.computedObject.getPath();
        const { trueValue } = this.item.attrs
        const value = trueValue

        return [
          {
            path,
            value,
            tag: {
              cmd: this.computedObject.getIdent(),
            }
          }
        ]
      },

      getPulseValue() {
        const path = this.computedObject.getPath();
        const { trueValue, falseValue } = this.item.attrs

        return [
          {
            path,
            value: falseValue,
          },
          {
            path,
            value: trueValue,
          },
          {
            path,
            value: falseValue,
            tag: {
              cmd: this.computedObject.getIdent(),
            }
          }
        ]
      },
      onEvent(ev) {
        const value = this.panel.getEventValue(ev)
        this.value = value;
        if (!this.firstEvent) {
          this.firstEvent = ev
        }
        this.prevEvent = ev
      },
      async emitValue() {
        let set = []
        switch (this.item.attrs.method) {
          case 'set':
            set = this.getSetValue()
            break
          case 'toggle':
            set = this.getToggleValue()
            break
          case 'pulse':
            if (this.item.attrs.pulseLength > 0) {
              this.emitPulseWithTimeout()
              return
            }
            set = this.getPulseValue()
            break
        }
        this.panel.emit(this.computedObject, {
          set,
        });

        this.design.bus.emit('close:set')
      },
      async emitPulseWithTimeout() {
        const path = this.computedObject.getPath();
        const { trueValue, falseValue, pulseLength } = this.item.attrs
        const setTrueValue = {
          path,
          value: trueValue,
        }

        const setFalseValue = {
          path,
          value: falseValue,
        }

        this.panel.emit(this.computedObject, {
            set: [
              setFalseValue,
              setTrueValue,
            ]
        });
        
        await new Promise(resolve => setTimeout(() => {
          this.panel.emit(this.computedObject, {
              set: [
                setFalseValue,
              ]
          });
          resolve()
        }, pulseLength));

        if (this.isSetObject) {
          this.design.bus.emit('close:set')
        }
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      let node = ''
      if (this.isSetObject) {
        node = {
          type: 'NgLoading',
          props: {
            modelValue: this.loading,
            label: 'send',
            spinner: true,
            spinnerIcon: 'comment',
            'onUpdate:modelValue': (val) => {
              this.loading = val
            },
          },
        }
      } else {
         node = {
          type: 'NgActionButton',
          props: {
            ...this.item.attrs,
            loading: this.loading,
            'onClick': () => {
              if (!this.item.attrs.readonly) {
                this.emitValue()
              }
            },
          },
        }
      }
      return [
        // any ui component node can be used...
        {
          type: 'div',
          props: {
          },
          children: {
            default: () => {
              return [
                node,
              ]
            }
          },
        },
      ]
    },
  }
})